import { Plugin, PluginType } from "@segment/analytics-react-native";
import { Instrumentation, Interaction, LocalData, Navigation, Network, RawSignal, SignalType } from "./types";
import { SignalsJS } from "./compat";
import { LivePlugins, SafeProxy, sandboxExecute } from "@segment/analytics-react-native-plugin-live";
export class SignalsPlugin extends Plugin {
  type = PluginType.after;
  _config = {
    maximumBufferSize: 1000,
    broadcastInterval: 60
  };
  set config(value) {
    this._config = value;
    this.updateJSConfiguration();
    this.updateNativeConfiguration();
  }
  get broadcasters() {
    return this._config.broadcasters ?? [];
  }
  _queuedSignals = [];
  _ready = false;
  _counter = 0;
  _timerStarted = false;
  _signals = new SignalsJS();
  _runtime = {};
  get runtime() {
    return {
      ...this._runtime
    };
  }
  get anonymousId() {
    return this.analytics?.userInfo.get().anonymousId ?? "";
  }
  get nextIndex() {
    return this._signals.getNextIndex();
  }
  get buffer() {
    return this._signals.signalBuffer;
  }
  get jsSignals() {
    return this._signals;
  }
  constructor() {
    super();
  }
  prepare(compatLayer) {
    this._compatLayer = compatLayer;
    this._runtime = {
      'SignalType': new Proxy(SignalType, new SafeProxy()),
      'RawSignal': new Proxy(RawSignal, new SafeProxy()),
      'NavigationAction': new Proxy(Navigation.Action, new SafeProxy()),
      'NavigationData': new Proxy(Navigation.Data, new SafeProxy()),
      'NavigationSignal': new Proxy(Navigation.Signal, new SafeProxy()),
      'InteractionData': new Proxy(Interaction.Data, new SafeProxy()),
      'InteractionSignal': new Proxy(Interaction.Signal, new SafeProxy()),
      'NetworkAction': new Proxy(Network.Action, new SafeProxy()),
      'NetworkData': new Proxy(Network.Data, new SafeProxy()),
      'NetworkSignal': new Proxy(Network.Signal, new SafeProxy()),
      'LocalDataAction': new Proxy(LocalData.Action, new SafeProxy()),
      'LocalData': new Proxy(LocalData.Data, new SafeProxy()),
      'LocalDataSignal': new Proxy(LocalData.Signal, new SafeProxy()),
      'EventType': new Proxy(Instrumentation.EventType, new SafeProxy()),
      'InstrumentationData': new Proxy(Instrumentation.Data, new SafeProxy()),
      'InstrumentationSignal': new Proxy(Instrumentation.Signal, new SafeProxy()),
      'Signals': new Proxy(SignalsPlugin, new SafeProxy()),
      'signals': this._signals
    };
  }
  readyToStart() {
    this._ready = true;
    this.updateJSConfiguration();
    this.updateNativeConfiguration();
    this.replayQueuedSignals();
  }
  static get instance() {
    return this._instance || (this._instance = new this());
  }
  configure(analytics) {
    this.analytics = analytics;
    const liveplugin = analytics.getPlugins().find(p => p instanceof LivePlugins);
    if (liveplugin) {
      liveplugin.addDependent(this);
    }
  }
  execute(event) {
    if (event.context?.__eventOrigin?.type == 'js') return;
    this.emit(new Instrumentation.Signal(event));
    return event;
  }
  emit(signal) {
    if (!this._ready) {
      this._queuedSignals.push(signal);
      return;
    }
    this.broadcasters.forEach(b => {
      b.added(signal);
    });
    this._signals.add(signal);
    if (this._compatLayer) {
      const params = {
        ...this.runtime,
        'signal': signal
      };
      sandboxExecute(this._compatLayer, 'processSignal(signal);', params);
    }
    if (this._counter > this._config.maximumBufferSize) {
      this._counter = 0;
      this.broadcasters.forEach(b => {
        b.relay();
      });
    }
  }
  replayQueuedSignals() {
    while (this._queuedSignals.length) {
      var signal = this._queuedSignals.shift();
      if (signal) {
        signal.index = this.nextIndex;
        signal.anonymousId = this.anonymousId;
        this.emit(signal);
      }
    }
  }
  updateJSConfiguration() {
    this._signals.maxBufferSize = this._config.maximumBufferSize;
  }
  updateNativeConfiguration() {
    this.schedule();
  }
  schedule() {
    if (this._timerStarted) return;
    this._timerStarted = true;
    this._timer = setInterval(() => {
      this.broadcasters.forEach(b => {
        b.relay();
      });
    }, this._config.broadcastInterval * 1000);
  }

  // @ts-ignore
  unschedule() {
    if (this._timerStarted) {
      clearTimeout(this._timer);
    }
  }
}
export const Signals = SignalsPlugin.instance;
//# sourceMappingURL=signals.js.map