import { checkResponseForErrors, chunk, createTrackEvent, defaultConfig, QueueFlushingPlugin, SegmentClient, SegmentEvent, translateHTTPError, uploadEvents } from "@segment/analytics-react-native";
import { RawSignal, SignalBroadcaster } from "src/types";

const MAX_EVENTS_PER_BATCH = 100;
const MAX_PAYLOAD_SIZE_IN_KB = 500;

export class SegmentBroadcaster implements SignalBroadcaster{
  
  private _analytics: SegmentClient;

  private sendEvents = async (events: SegmentEvent[]): Promise<void> => {
    if (events.length === 0) {
      return Promise.resolve();
    }

    // We're not sending events until Segment has loaded all settings
    // await this.settingsPromise;

    const config = this._analytics.getConfig() ?? defaultConfig;

    const chunkedEvents: SegmentEvent[][] = chunk(
      events,
      config.maxBatchSize ?? MAX_EVENTS_PER_BATCH,
      MAX_PAYLOAD_SIZE_IN_KB
    );

    let sentEvents: SegmentEvent[] = [];
    let numFailedEvents = 0;

    await Promise.all(
      chunkedEvents.map(async (batch: SegmentEvent[]) => {
        try {
          const res = await uploadEvents({
            writeKey: config.writeKey,
            url: 'https://signals.segment.io/v1/b',
            events: batch,
          });
          checkResponseForErrors(res);
          sentEvents = sentEvents.concat(batch);
        } catch (e) {
          this._analytics.reportInternalError(translateHTTPError(e));
          this._analytics.logger.warn(e);
          numFailedEvents += batch.length;
        } finally {
          await this._flushPlugin.dequeue(sentEvents);
        }
      })
    );

    if (sentEvents.length) {
      if (config.debug === true) {
        this._analytics.logger.info(`Sent ${sentEvents.length} signlas`);
      }
    }

    if (numFailedEvents) {
      this._analytics.logger.error(`Failed to send ${numFailedEvents} signals.`);
    }

    return Promise.resolve();
  };
  
  private _flushPlugin: QueueFlushingPlugin = new QueueFlushingPlugin(this.sendEvents, 'signals');

  constructor(analytics: SegmentClient) {
    this._analytics = analytics;
    this._flushPlugin.configure(analytics);
  }
  
  added<T>(signal: RawSignal<T>): void {
    // TODO: add obfuscation
    const userInfo = this._analytics.userInfo.get();
    this._flushPlugin.execute(this.createMiniTrackEvent(
      userInfo.anonymousId,
      userInfo.userId,
      signal
    ))
  }

  relay(): void {
    this._flushPlugin.flush();
  }

  createMiniTrackEvent(anonymousId: string, userId?: string, properties?: {}): SegmentEvent {
    const miniEvent = createTrackEvent({
      event: 'Segment Signal Generated',
      properties: properties,
    });
    miniEvent.anonymousId = anonymousId;
    miniEvent.userId = userId;
    miniEvent.timestamp = new Date().toISOString();
    miniEvent.context = {};
    miniEvent.integrations = {};
    return miniEvent;
  }
}