import { SegmentEvent } from '@segment/analytics-react-native';
import { Signals } from './signals';

export type Config = {
  maximumBufferSize: number;
  broadcastInterval: number;
  broadcasters?: SignalBroadcaster[];
}

export abstract class RawSignal<T> {
  abstract anonymousId: string;
  abstract type: string;
  abstract timestamp: string;
  abstract index: number;
  abstract data: T;
}

export enum SignalType {
  Interaction = 'interaction',
  Navigation = 'navigation',
  Network = 'network',
  LocalData = 'localData',
  Instrumentation = 'instrumentation',
  UserDefined = 'userDefined',
}

export interface SignalBroadcaster {
  added<T>(signal: RawSignal<T>): void;
  relay(): void;
}

export namespace Navigation {
  export enum Action {
    Forward = 'forward',
    Backward = 'backward',
    Modal = 'modal',
    Entering = 'entering',
    Leaving = 'leaving',
    Page = 'page',
    Popup = 'popup',
  }

  export class Data {
    readonly action: Action;
    readonly screen: string;

    constructor(action: Action, screen: string) {
      this.action = action;
      this.screen = screen;
    }
  }

  export class Signal extends RawSignal<Data> {
    constructor(action: Action, screen: string) {
      super();
      this.data = new Data(action, screen);
    }

    anonymousId: string = Signals.anonymousId;
    type: string = SignalType.Navigation;
    timestamp: string = new Date().toISOString();
    index: number = Signals.nextIndex;
    data: Data;
  }
}

export namespace Interaction {
  export class Data {
    readonly component: string;
    readonly title?: string;
    readonly data?: object;

    constructor(component: string, title?: string, data?: object) {
      this.component = component;
      this.title = title;
      this.data = data;
    }
  }

  export class Signal extends RawSignal<Data> {
    constructor(control: string, title?: string, data?: string) {
      super();
      const json =
        data === null || data === undefined ? undefined : JSON.parse(data);
      this.data = new Data(control, title, json);
    }

    anonymousId: string = Signals.anonymousId;
    type: string = SignalType.Interaction;
    timestamp: string = new Date().toISOString();
    index: number = Signals.nextIndex;
    data: Data;
  }
}

export namespace Network {
  export enum Action {
    Request = 'request',
    Response = 'response',
  }

  export class Data {
    readonly action: Action;
    readonly url: string;
    readonly data?: object;

    constructor(action: Action, url: string, data?: object) {
      this.action = action;
      this.url = url;
      this.data = data;
    }
  }

  export class Signal extends RawSignal<Data> {
    constructor(action: Action, url: string, data?: string) {
      super();
      const json =
        data === null || data === undefined ? undefined : JSON.parse(data);
      this.data = new Data(action, url, json);
    }

    anonymousId: string = Signals.anonymousId;
    type: string = SignalType.Network;
    timestamp: string = new Date().toISOString();
    index: number = Signals.nextIndex;
    data: Data;
  }
}

export namespace LocalData {
  export enum Action {
    Loaded = 'loaded',
    Updated = 'updated',
    Saved = 'saved',
    Deleted = 'deleted',
    Undefined = 'undefined',
  }

  export class Data {
    readonly action: Action;
    readonly identifier: string;
    readonly data?: object;

    constructor(action: Action, identifier: string, data?: object) {
      this.action = action;
      this.identifier = identifier;
      this.data = data;
    }
  }

  export class Signal extends RawSignal<Data> {
    constructor(action: Action, identifier: string, data?: string) {
      super();
      const json =
        data === null || data === undefined ? undefined : JSON.parse(data);
      this.data = new Data(action, identifier, json);
    }

    anonymousId: string = Signals.anonymousId;
    type: string = SignalType.LocalData;
    timestamp: string = new Date().toISOString();
    index: number = Signals.nextIndex;
    data: Data;
  }
}

export namespace Instrumentation {
  export enum EventType {
    Track = 'track',
    Screen = 'screen',
    Alias = 'alias',
    Identify = 'identify',
    Group = 'group',
    Unknown = 'unknown',
  }

  export class Data {
    readonly type: EventType;
    readonly rawEvent?: object;

    constructor(type: EventType, rawEvent?: object) {
      this.type = type;
      this.rawEvent = rawEvent;
    }
  }

  export class Signal extends RawSignal<Data> {
    constructor(event: SegmentEvent) {
      super();
      const type: string = event.type;
      this.data = new Data(type as EventType, event);
    }

    anonymousId: string = Signals.anonymousId;
    type: string = SignalType.Instrumentation;
    timestamp: string = new Date().toISOString();
    index: number = Signals.nextIndex;
    data: Data;
  }
}
